module.exports = {
    "extends": "eslint:recommended",
    "env": {
        "node": true
    },
    "rules": {
        "indent": ["error", 4],
        "linebreak-style": ["error", "unix"],
        "semi": ["error", "always"],
        "no-cond-assign": ["error", "always"]
    }
}
